/*
 *  Copyright 2016-2019 Netflix, Inc.
 *
 *     Licensed under the Apache License, Version 2.0 (the "License");
 *     you may not use this file except in compliance with the License.
 *     You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 *
 */
package com.netflix.hollow.core.write.objectmapper;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Indicates the hollow type name of a POJO or the hollow field name of a field
 * declared by a POJO.
 * <p>
 * This annotation may be used to override the hollow class name derived from the
 * name of a {@code Class} of a POJO, or override the hollow field name derived
 * from the name of a {@code Field} declared by a POJO.
 */
@Retention(RetentionPolicy.RUNTIME)
@Target( {ElementType.TYPE, ElementType.FIELD})
public @interface HollowTypeName {

    /**
     * Returns the hollow type name or hollow field name.
     *
     * @return the hollow type name or hollow field name
     */
    String name();
}
